-- ------------------------------------------------------------------------------ --
--                                TradeSkillMaster                                --
--                          https://tradeskillmaster.com                          --
--    All Rights Reserved - Detailed license information included with addon.     --
-- ------------------------------------------------------------------------------ --

local TSM = select(2, ...) ---@type TSM
local PlayerGoldText = TSM.UI.Views:NewPackage("PlayerGoldText") ---@type AddonPackage
local L = TSM.Include("Locale").GetTable()
local Event = TSM.LibTSMWoW:Include("Service.Event")
local TempTable = TSM.LibTSMUtil:Include("BaseType.TempTable")
local Money = TSM.LibTSMUtil:Include("UI.Money")
local SessionInfo = TSM.LibTSMWoW:Include("Util.SessionInfo")
local UIElements = TSM.Include("UI.UIElements")
local Tooltip = TSM.Include("UI.Tooltip")
local private = {
	settings = nil,
}



-- ============================================================================
-- Module Functions
-- ============================================================================

function PlayerGoldText.OnInitialize(settingsDB)
	private.settings = settingsDB:NewView()
		:AddKey("global", "coreOptions", "regionWide")
		:AddKey("global", "appearanceOptions", "showTotalMoney")
		:AddKey("sync", "internalData", "money")
end

---Creates a new player gold text element.
---@param id string The id to assign to the element
---@return Button @The element
function PlayerGoldText.New(id)
	local text = UIElements.New("Button", id)
		:SetJustifyH("RIGHT")
		:SetFont("TABLE_TABLE1")
		:SetText(private.GetText())
		:SetTooltip(private.TooltipFunc)
	text:AddCancellable(Event.GetPublisher("PLAYER_MONEY")
		:MapWithFunction(private.GetText)
		:CallMethod(text, "SetText")
	)
	return text
end



-- ============================================================================
-- Private Helper Functions
-- ============================================================================

function private.GetText()
	local amount = private.settings.money
	if private.settings.showTotalMoney then
		for _, money, character, factionrealm in private.settings:AccessibleValueIterator("money", not private.settings.regionWide) do
			if money > 0 and not SessionInfo.IsPlayer(character, factionrealm) then
				amount = amount + money
			end
		end
	end
	return Money.ToStringForUI(amount)
end

function private.TooltipFunc()
	local tooltipLines = TempTable.Acquire()
	local playerMoney = private.settings.money
	local total = playerMoney
	tinsert(tooltipLines, private.GetTooltipLine(SessionInfo.GetCharacterName(), playerMoney))
	local numPosted, numSold, postedGold, soldGold = TSM.MyAuctions.GetAuctionInfo()
	if numPosted then
		tinsert(tooltipLines, private.GetTooltipLine(format("  "..L["%s Sold Auctions"], numSold), soldGold))
		tinsert(tooltipLines, private.GetTooltipLine(format("  "..L["%s Posted Auctions"], numPosted), postedGold))
	end
	for _, money, character, factionrealm in private.settings:AccessibleValueIterator("money", not private.settings.regionWide) do
		if money > 0 and not SessionInfo.IsPlayer(character, factionrealm) then
			character = SessionInfo.FormatCharacterName(character, factionrealm)
			tinsert(tooltipLines, private.GetTooltipLine(character, money))
			total = total + money
		end
	end
	tinsert(tooltipLines, 1, private.GetTooltipLine(L["Total Gold"], total))
	return strjoin("\n", TempTable.UnpackAndRelease(tooltipLines))
end

function private.GetTooltipLine(label, value)
	return strjoin(Tooltip.GetSepChar(), label..":", Money.ToStringForUI(value))
end
